--RESTORE BackupSample from our base backup.
USE MASTER
GO
RESTORE DATABASE BackupSample FROM DISK = 'C:\KNOWBACKUPS\BACKUPSAMPLE_BASE.BAK' WITH NORECOVERY, REPLACE
RESTORE DATABASE BackupSample WITH RECOVERY

--Clear backup history just in case I have ran this demo before.
USE MSDB
GO
DECLARE @DATE DATETIME
SET @DATE = GETDATE()+1
EXEC SP_DELETE_BACKUPHISTORY @DATE
SELECT * FROM msdb.dbo.BACKUPSET

--Show the top 100 orders by SalesOrderID DESC
--This will allow us to see when we insert new records
USE MASTER
GO
USE BACKUPSAMPLE
GO
select top 100 *
from Sales order by SalesOrderID DESC

--Backup the database and log 
BACKUP DATABASE BACKUPSAMPLE TO DISK = 'C:\KNOWBACKUPS\BACKUPS\BACKUPSAMPLE_BASE.BAK'
BACKUP LOG BACKUPSAMPLE TO DISK = 'C:\KNOWBACKUPS\BACKUPS\BACKUPSAMPLE_LOG1.BAK'

--INSERT a single record into the sales table.
INSERT INTO [BackupSample].[dbo].[sales]
           ([SalesOrderID]
           ,[CarrierTrackingNumber]
           ,[OrderQty]
           ,[ProductID]
           ,[SpecialOfferID]
           ,[UnitPrice]
           ,[UnitPriceDiscount]
           ,[LineTotal]
           ,[rowguid]
           ,[ModifiedDate])
     VALUES
           ('75150'
           ,NULL
           ,'2'
           ,'879'
           ,'1'
           ,'159.00'
           ,'0.00'
           ,'318.00'
           ,NEWID()
           ,getdate())
GO

--Backup the log 
BACKUP LOG BACKUPSAMPLE TO DISK = 'C:\KNOWBACKUPS\BACKUPS\BACKUPSAMPLE_LOG2.BAK'

--INSERT another record into the sales table.
INSERT INTO [BackupSample].[dbo].[sales]
           ([SalesOrderID]
           ,[CarrierTrackingNumber]
           ,[OrderQty]
           ,[ProductID]
           ,[SpecialOfferID]
           ,[UnitPrice]
           ,[UnitPriceDiscount]
           ,[LineTotal]
           ,[rowguid]
           ,[ModifiedDate])
     VALUES
           ('75200'
           ,NULL
           ,'2'
           ,'879'
           ,'1'
           ,'159.00'
           ,'0.00'
           ,'318.00'
           ,NEWID()
           ,getdate())
GO

--Take another log backup.
BACKUP LOG BACKUPSAMPLE TO DISK = 'C:\KNOWBACKUPS\BACKUPS\BACKUPSAMPLE_LOG3.BAK'

--Lets insert another record for good measure into the sales table.
INSERT INTO [BackupSample].[dbo].[sales]
           ([SalesOrderID]
           ,[CarrierTrackingNumber]
           ,[OrderQty]
           ,[ProductID]
           ,[SpecialOfferID]
           ,[UnitPrice]
           ,[UnitPriceDiscount]
           ,[LineTotal]
           ,[rowguid]
           ,[ModifiedDate])
     VALUES
           ('75250'
           ,NULL
           ,'2'
           ,'879'
           ,'1'
           ,'159.00'
           ,'0.00'
           ,'318.00'
           ,NEWID()
           ,getdate())
GO

--A final log backup.
BACKUP LOG BACKUPSAMPLE TO DISK = 'C:\KNOWBACKUPS\BACKUPS\BACKUPSAMPLE_LOG4.BAK'

--Now lets see if we see our 3 records we inserted.
--Should have SalesOrderID 75150, 75200 and 75250
USE BACKUPSAMPLE
GO
select top 100 *
from Sales order by SalesOrderID DESC

/* Now lets run a simple script to generate a restore script
that includes the last FULL backup and each transaction log
since the last FULL backup, this can be a life saver */
--BEGIN SCRIPT

DECLARE @databaseName sysname
DECLARE @backupStartDate datetime
DECLARE @backup_set_id_start INT
DECLARE @backup_set_id_end INT

-- set database to be used
SET @databaseName = 'BackupSample' 

SELECT @backup_set_id_start = MAX(backup_set_id) 
FROM  msdb.dbo.backupset 
WHERE database_name = @databaseName AND type = 'D' and is_copy_only = '0'

SELECT @backup_set_id_end = MIN(backup_set_id) 
FROM  msdb.dbo.backupset 
WHERE database_name = @databaseName AND type = 'D' and is_copy_only = '0'
AND backup_set_id > @backup_set_id_start

IF @backup_set_id_end IS NULL SET @backup_set_id_end = 2147483647

SELECT backup_set_id, 'RESTORE DATABASE ' + @databaseName + ' FROM DISK = ''' 
               + mf.physical_device_name + ''' WITH NORECOVERY'
FROM    msdb.dbo.backupset b,
           msdb.dbo.backupmediafamily mf
WHERE    b.media_set_id = mf.media_set_id
           AND b.database_name = @databaseName
          AND b.backup_set_id = @backup_set_id_start
UNION
SELECT backup_set_id, 'RESTORE LOG ' + @databaseName + ' FROM DISK = ''' 
               + mf.physical_device_name + ''' WITH NORECOVERY'
FROM    msdb.dbo.backupset b,
           msdb.dbo.backupmediafamily mf
WHERE    b.media_set_id = mf.media_set_id
           AND b.database_name = @databaseName
          AND b.backup_set_id >= @backup_set_id_start AND b.backup_set_id < @backup_set_id_end
          AND b.type = 'L'
UNION
SELECT 2147483647 AS backup_set_id, 'RESTORE DATABASE ' + @databaseName + ' WITH RECOVERY'
ORDER BY backup_set_id

--END RESTORE SCRIPT
