--RESTORE WITH DIFFERENTIAL

--RESTORE BackupSample from our base backup.
USE MASTER
GO
RESTORE DATABASE BackupSample FROM DISK = 'C:\KNOWBACKUPS\BACKUPSAMPLE_BASE.BAK' WITH NORECOVERY, REPLACE
RESTORE DATABASE BackupSample WITH RECOVERY

--Clear backup history just in case I have ran this demo before.
USE MSDB
GO
DECLARE @DATE DATETIME
SET @DATE = GETDATE()+1
EXEC SP_DELETE_BACKUPHISTORY @DATE
SELECT * FROM msdb.dbo.BACKUPSET
USE master
GO

--BACKUP FOR CLEAN LIST
BACKUP DATABASE BackupSample TO DISK = 'C:\KNOWBACKUPS\BACKUPS\BACKUPSAMPLE_FULL_DEMODIFF.BAK'
BACKUP LOG BackupSample TO DISK = 'C:\KNOWBACKUPS\BACKUPS\BACKUPSAMPLE_DEMODIFF_LOG.BAK'

--Insert First Record
INSERT INTO [BackupSample].[dbo].[sales]
           ([SalesOrderID]
           ,[CarrierTrackingNumber]
           ,[OrderQty]
           ,[ProductID]
           ,[SpecialOfferID]
           ,[UnitPrice]
           ,[UnitPriceDiscount]
           ,[LineTotal]
           ,[rowguid]
           ,[ModifiedDate])
     VALUES
           ('75150'
           ,NULL
           ,'2'
           ,'879'
           ,'1'
           ,'159.00'
           ,'0.00'
           ,'318.00'
           ,NEWID()
           ,getdate())
GO
--LOG BACKUP
BACKUP LOG BACKUPSAMPLE TO DISK = 'C:\KNOWBACKUPS\BACKUPS\BACKUPSAMPLE_DEMODIFF_LOG1.BAK'

--Insert second record
INSERT INTO [BackupSample].[dbo].[sales]
           ([SalesOrderID]
           ,[CarrierTrackingNumber]
           ,[OrderQty]
           ,[ProductID]
           ,[SpecialOfferID]
           ,[UnitPrice]
           ,[UnitPriceDiscount]
           ,[LineTotal]
           ,[rowguid]
           ,[ModifiedDate])
     VALUES
           ('75200'
           ,NULL
           ,'2'
           ,'879'
           ,'1'
           ,'159.00'
           ,'0.00'
           ,'318.00'
           ,NEWID()
           ,getdate())
GO

--PERFORM DIFFERENTIAL
BACKUP DATABASE BackupSample TO DISK = 'C:\KNOWBACKUPS\backups\BACKUPSAMPLE_DIFF.BAK' WITH DIFFERENTIAL

--Insert a third record

INSERT INTO [BackupSample].[dbo].[sales]
           ([SalesOrderID]
           ,[CarrierTrackingNumber]
           ,[OrderQty]
           ,[ProductID]
           ,[SpecialOfferID]
           ,[UnitPrice]
           ,[UnitPriceDiscount]
           ,[LineTotal]
           ,[rowguid]
           ,[ModifiedDate])
     VALUES
           ('75400'
           ,NULL
           ,'2'
           ,'879'
           ,'1'
           ,'159.00'
           ,'0.00'
           ,'318.00'
           ,NEWID()
           ,getdate())
GO

--Backup Log
BACKUP LOG BACKUPSAMPLE TO DISK = 'C:\KNOWBACKUPS\BACKUPS\BACKUPSAMPLE_DEMODIFF_LOG3.BAK'

--Insert a fourth record.
INSERT INTO [BackupSample].[dbo].[sales]
           ([SalesOrderID]
           ,[CarrierTrackingNumber]
           ,[OrderQty]
           ,[ProductID]
           ,[SpecialOfferID]
           ,[UnitPrice]
           ,[UnitPriceDiscount]
           ,[LineTotal]
           ,[rowguid]
           ,[ModifiedDate])
     VALUES
           ('75500'
           ,NULL
           ,'2'
           ,'879'
           ,'1'
           ,'159.00'
           ,'0.00'
           ,'318.00'
           ,NEWID()
           ,getdate())
GO

--Final Transaction log backup
BACKUP LOG BACKUPSAMPLE TO DISK = 'C:\KNOWBACKUPS\BACKUPS\BACKUPSAMPLE_DEMODIFF_LOG4.BAK'

/* Now lets run a simple script to generate a restore script
that includes the last FULL backup, then the last Differential backup
plus each transaction log since the last Differential backup, 
this can be a life saver */
--BEGIN SCRIPT

/* This script uses msdb to gather your last full,
 differential, and transaction log backups to 
generate your restore scripts*/
--BEGIN SCRIPT

--Declare the variables we will be using.

DECLARE @db_name NVARCHAR(128)
DECLARE @backupstarttime DATETIME
DECLARE @backupsetbegin INT
DECLARE @backupsetfullbackup INT
DECLARE @backupsetcomplete INT

--Type the name of the database you want to create the scripts for.
SET @db_name = 'BackupSample' 

--Gets the most recent backup_set_id for the full backup. 
--Notice we are excluding any copy only backups
SELECT  @backupsetfullbackup = MAX(backup_set_id)
FROM    msdb.dbo.backupset
WHERE   database_name = @db_name
        AND type = 'D'and is_copy_only = '0'

--Gets the most recent backup_set_id for the differential backup.
SELECT  @backupsetbegin = MAX(backup_set_id)
FROM    msdb.dbo.backupset
WHERE   database_name = @db_name
        AND type = 'I'

SELECT  @backupsetcomplete = MIN(backup_set_id)
FROM    msdb.dbo.backupset
WHERE   database_name = @db_name
        AND type = 'D' and is_copy_only = '0'
        AND backup_set_id > @backupsetbegin

IF @backupsetcomplete IS NULL SET @backupsetcomplete = 2147483647

--Creates your full backup restore script.
SELECT  backup_set_id ,
        'RESTORE DATABASE ' + @db_name + ' FROM DISK = '''
        + bmf.physical_device_name + ''' WITH NORECOVERY' AS Script
FROM    msdb.dbo.backupset bs
        JOIN msdb.dbo.backupmediafamily bmf ON bs.media_set_id = bmf.media_set_id
WHERE   bs.media_set_id = bmf.media_set_id
        AND bs.database_name = @db_name
        AND bs.backup_set_id = @backupsetfullbackup
UNION
--Creates your differential restore script.
SELECT  backup_set_id ,
        'RESTORE DATABASE ' + @db_name + ' FROM DISK = '''
        + bmf.physical_device_name + ''' WITH NORECOVERY' AS Script
FROM    msdb.dbo.backupset bs
        JOIN msdb.dbo.backupmediafamily bmf ON bs.media_set_id = bmf.media_set_id
WHERE   bs.media_set_id = bmf.media_set_id
        AND bs.database_name = @db_name
        AND bs.backup_set_id = @backupsetbegin
UNION
--Creates your transaction log restore scripts.
SELECT  backup_set_id ,
        'RESTORE LOG ' + @db_name + ' FROM DISK = '''
        + bmf.physical_device_name + ''' WITH NORECOVERY' AS Script
FROM    msdb.dbo.backupset bs
        JOIN msdb.dbo.backupmediafamily bmf ON bs.media_set_id = bmf.media_set_id
WHERE   bs.media_set_id = bmf.media_set_id
        AND bs.database_name = @db_name
        AND bs.backup_set_id >= @backupsetbegin
        AND bs.backup_set_id < @backupsetcomplete
        AND bs.type = 'L'
UNION
--Creates the script to bring the database online.
SELECT  2147483647 AS backup_set_id ,
        'RESTORE DATABASE ' + @db_name + ' WITH RECOVERY' AS Script
ORDER BY backup_set_id

--END SCRIPT

