--RESTORE BackupSample from our base backup.
USE MASTER
GO
RESTORE DATABASE BACKUPFILEGROUPSAMPLE FROM DISK = 'C:\KNOWBACKUPS\BACKUPS\BackupFileGroupSample_BASE.bak' WITH NORECOVERY, REPLACE
RESTORE DATABASE BACKUPFILEGROUPSAMPLE WITH RECOVERY

--Change database and view data in table
USE BACKUPFILEGROUPSAMPLE
GO
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
FROM    Sales
WHERE   modifieddate < '2013-01-02'

--Can view just dates for fun
USE BackupFilegroupSample
GO
SELECT MODIFIEDDATE FROM SALES

--Need a Full Backup then a Log backup in order to add FileGroups.
BACKUP DATABASE backupfilegroupsample TO DISK = 'c:\knowbackups\backups\BackupFileGroupSample_BASE.bak'
BACKUP LOG BACKUPFILEGROUPSAMPLE TO DISK = 'NUL'

--Create File Groups  **SHOW CURRENT FILES AND FILEGROUPS IN GUI**
USE MASTER
GO
ALTER DATABASE [BACKUPFILEGROUPSAMPLE] ADD FILEGROUP [2011]
ALTER DATABASE [BACKUPFILEGROUPSAMPLE] ADD FILEGROUP [2010]
ALTER DATABASE [BACKUPFILEGROUPSAMPLE] ADD FILEGROUP [2009]

--Create Files and add to File Groups  **SHOW FILES GROUPS CREATE - NO FILES**
USE MASTER
GO
ALTER DATABASE backupfilegroupsample
ADD FILE
( NAME = BackupFilegroupSample2009,
	FILENAME = 'C:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\BackupFilegroupSample2009.ndf',
	SIZE = 1MB,
	MAXSIZE = 100MB,
	FILEGROWTH = 1MB
	)TO FILEGROUP [2009];
	GO
ALTER DATABASE backupfilegroupsample
ADD FILE
( NAME = BackupFilegroupSample2010,
	FILENAME = 'C:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\BackupFilegroupSample2010.ndf',
	SIZE = 1MB,
	MAXSIZE = 100MB,
	FILEGROWTH = 1MB
	)TO FILEGROUP [2010];
	GO
ALTER DATABASE backupfilegroupsample
ADD FILE
( NAME = BackupFilegroupSample2011,
	FILENAME = 'C:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\BackupFilegroupSample2011.ndf',
	SIZE = 1MB,
	MAXSIZE = 100MB,
	FILEGROWTH = 1MB
	)TO FILEGROUP [2011];
	GO
	
--**SHOW FILE GROUPS, FILES, SHOW IN EXPLORER THE FILE SIZE OF 1MB**
--CREATE A PARTITION FUNCTION TO RELOCATE THE DATA
USE MASTER
GO
USE [BackupFilegroupSample]
GO
BEGIN TRANSACTION
CREATE PARTITION FUNCTION [SalesModifiedDateFunction](DATETIME) AS RANGE LEFT FOR VALUES (N'2010-01-01T00:00:00', N'2011-01-01T00:00:00', N'2012-01-01T00:00:00', N'2013-01-01T00:00:00')

CREATE PARTITION SCHEME [SalesModifiedDateScheme] AS PARTITION [SalesModifiedDateFunction] TO ([2009], [2010], [2011], [PRIMARY], [PRIMARY])

CREATE CLUSTERED INDEX [ClusteredIndex_on_SalesModifiedDateScheme_634683074366356000] ON [dbo].[SALES] 
(
[ModifiedDate]
)WITH (SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF) ON [SalesModifiedDateScheme]([ModifiedDate])

DROP INDEX [ClusteredIndex_on_SalesModifiedDateScheme_634683074366356000] ON [dbo].[SALES] WITH ( ONLINE = OFF )

COMMIT TRANSACTION

--@@@@@ END PARTITION FUNCTION

--Now let's work on a piecemeal restore  **** This is where things get fun ****

--Get a baseline backup of our partitioned database
BACKUP DATABASE BackupFileGroupSample TO DISK = 'C:\KnowBackups\Backups\BackupFileGroupSample_After.bak'

--Backup filegroups
BACKUP DATABASE backupfilegroupsample
	FILEGROUP = 'PRIMARY',
	FILEGROUP = '2009',
	FILEGROUP = '2010',
	FILEGROUP = '2011'
	TO DISK = 'C:\KNOWBACKUPS\Backups\FILEGROUP_BACKUP.BAK'

--View Data to show all years are available
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
FROM    Sales 
WHERE modifieddate < '2013-01-02'

--Restore only the primary file group.
USE master
GO
RESTORE DATABASE BackupFileGroupSample FILEGROUP = 'Primary' 
FROM DISK = 'C:\KnowBackups\backups\BackupFileGroupSample_After.bak' WITH Partial, RECOVERY, REPLACE

--Show that only the primary file group and the transaction log are online.
SELECT  NAME ,
        state_desc
FROM    BackupFileGroupSample.sys.database_files

--Can now view information in the online filegroups
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
FROM BACKUPFILEGROUPSAMPLE.dbo.Sales 
WHERE modifieddate > '2012-01-02'

--Can NOT access data in the recovery_pending filegroups
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
FROM BACKUPFILEGROUPSAMPLE.dbo.Sales 
WHERE modifieddate > '2009-01-02'

--RESTORE 2009 FILEGROUP
RESTORE DATABASE BackupFileGroupSample FILEGROUP = '2009'
WITH RECOVERY

--View State
SELECT  NAME ,
        state_desc
FROM    BackupFileGroupSample.sys.database_files

--QUERY 2009 Data Only
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
FROM BACKUPFILEGROUPSAMPLE.dbo.Sales 
WHERE modifieddate BETWEEN '2009-01-02' AND '2009-12-31' 

--QUERY More than 2009
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
FROM BACKUPFILEGROUPSAMPLE.dbo.Sales 
WHERE modifieddate > '2009-01-02'

--QUERY 2009 Data and 2012 Data
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
FROM BACKUPFILEGROUPSAMPLE.dbo.Sales 
WHERE modifieddate BETWEEN '2009-01-02' AND '2009-12-31' 
		OR ModifiedDate BETWEEN '2012-01-02' AND '2012-12-31'

--Restore Other File Groups
RESTORE DATABASE BackupFileGroupSample FILEGROUP = '2010'
WITH RECOVERY

RESTORE DATABASE BackupFileGroupSample FILEGROUP = '2011'
WITH RECOVERY

--View State
SELECT  NAME ,
        state_desc
FROM    BackupFileGroupSample.sys.database_files

--View All Data
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
from BACKUPFILEGROUPSAMPLE.dbo.Sales where modifieddate < '2013-01-02'
